// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#pragma once

//------------------------------------------------------------------------------

#include "Event.h"

#include "WSAInterface.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * A mixin for sockets with some commong functons
 */
template <class Super>
class SocketMixin : public Super
{
public:
    using Super::Super;

    /**
     * Set the socket to non-blocking mode for the given events,
     * if it has a waiter
     */
    bool setNonBlocking(Event& event, long eventMask);

    /**
     * Add the given event to our waiter, if any.
     */
    void addToWaiter(Event& event);
};

//------------------------------------------------------------------------------
// Template definitions
//------------------------------------------------------------------------------

template <class Super>
inline bool SocketMixin<Super>::setNonBlocking(Event& event, long eventMask)
{
    if (Super::waiter!=nullptr) {
        return WSAInterface::get().eventSelect(Super::handle,
                                               event.getHandle(), eventMask)==0;
    } else {
        return true;
    }
}

//------------------------------------------------------------------------------

template <class Super>
inline void SocketMixin<Super>::addToWaiter(Event& event)
{
    if (Super::waiter!=nullptr) {
        event.addTo(*Super::waiter);
    }
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
